﻿using SFML.Graphics;
using SFML.Window;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BobbleCrowd
{
    class AnimationSprite
    {
        private String fileName;
        private int frameIndex;
        private int frameNum;
        private float run_FrameTime;
        private float max_FrameTime;
        private Sprite sprite;
        private int row;
        private int maxRow;
        public float scale;
        public Vector2f position;
        public float rotation;
        public Color color = Color.White;

        public AnimationSprite(String fileName, float animationTime) : this(fileName,0,1,animationTime)
        {}
        public AnimationSprite(String fileName, float animationTime, float scale) : this(fileName, 0, 1, animationTime,scale)
        {}
        public AnimationSprite(String fileName, int row, int maxRow, float animationTime) : this (fileName, row, maxRow, animationTime, new Vector2f(0,0),1)
        {}
        public AnimationSprite(String fileName, int row, int maxRow, float animationTime, float scale) : this(fileName, row, maxRow, animationTime, new Vector2f(0, 0), scale)
        {}
        public AnimationSprite(String fileName, int row, int maxRow, float animationTime, Vector2f position, float scale)
        {
            this.fileName = fileName;
            this.max_FrameTime = animationTime;
            this.row = row;
            this.maxRow = maxRow;

            this.position = position;
            this.scale = scale;

            rotation = 0;
            

            if (maxRow < 1)
                throw new Exception("zero frame row. Must be greater than 0");


        }
        public void Update(GameTime time)
        {

            this.run_FrameTime += (float)time.ElapsedTime.TotalMilliseconds;

            if(this.run_FrameTime>max_FrameTime)
            {
                run_FrameTime = 0;
                frameIndex++;
                frameIndex %= frameNum;
                sprite.TextureRect = new IntRect(frameIndex * (int)(sprite.Texture.Size.Y/maxRow), (int)((sprite.Texture.Size.Y/maxRow)*row), (int)(sprite.Texture.Size.Y/maxRow), (int)(sprite.Texture.Size.Y/maxRow));
            }
            sprite.Scale = new Vector2f(scale, scale);
            sprite.Position = this.position;
            sprite.Rotation = rotation;
            sprite.Color = color;

           
        }

        public void Draw(SFML.Graphics.RenderTarget target)
        {
           
            target.Draw(sprite);
        }

        public void Init()
        {
            sprite = new Sprite(Game.Content.Load<Texture>(fileName));
            sprite.Texture.Smooth = true;
            frameNum = (int)(sprite.Texture.Size.X / (sprite.Texture.Size.Y/maxRow));


            this.frameIndex = 0;
            this.run_FrameTime = 0;

            sprite.Origin = new Vector2f((sprite.Texture.Size.Y / (maxRow * 2)), (sprite.Texture.Size.Y / (maxRow * 2)));
            

            sprite.TextureRect = new IntRect(frameIndex * (int)(sprite.Texture.Size.Y / maxRow), (int)((sprite.Texture.Size.Y / maxRow) * row), (int)(sprite.Texture.Size.Y / maxRow), (int)(sprite.Texture.Size.Y / maxRow));

            frameIndex = MathUtil.Rand.Next((int)(sprite.Texture.Size.X / frameNum));
            run_FrameTime = MathUtil.Rand.NextFloat(0, max_FrameTime);

            sprite.Position = position;
            sprite.Scale = new Vector2f(scale, scale);
            sprite.Color = color;

            
        
        }


        public void SetScale(float scale)
        {
            sprite.Scale = new Vector2f(scale, scale);
        }

        public void SetColor(Color color)
        {
            this.color = color;
            this.sprite.Color = color;
        }
    }
}
