﻿using BobbleCrowd.Components;
using BobbleCrowd.Entities;
using SFML.Graphics;
using SFML.Window;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BobbleCrowd
{
    public class Bobble : Entity
    {

        private float boom_time_current;

        public BobbleInfo info;
        private float lifeTime;



       
        public Bobble(String name, ACollider collider, Vector2f pos, BobbleInfo info, float scale, bool createThinker) 
            : base(name, collider, pos)
        {
            this.info = info;

            if (createThinker)
            {
                MultiThinker thinker = new MultiThinker();
                Thinker = thinker;
                thinker.AddThinker(new RandomMover(1.0f, 2.0, MathUtil.TWO_PI, 0.2, MathUtil.TWO_PI * MathUtil.Rand.NextFloat()));
            }
          

            Category = CollisionGroup.Group2;

            Mask = CollisionGroup.Group1 | CollisionGroup.Group2 | CollisionGroup.Environment;

            Response = new PushResponse(true, 0.5f);

            Drawer = new BobbleDraw(scale, info);
            //bobble.Drawer = new BasicTextureDrawer(Game.Content.Load<Texture>("gfx/bodySprite.png"), false, true, scale, new IntRect(0, 0, 128, 128));
        }

        public Bobble(String name, ACollider collider, Vector2f pos, BobbleInfo info, float scale, bool createThinker, double lifeTime)
            : this(name, collider, pos, info, scale, createThinker)
        {
            this.boom_time_current = (float)(lifeTime * 1000.0);    
        }

        public override void Update()
        {
            base.Update();

            if (World.mode is BobbleSortMode)
            {
                if (Category == Entities.CollisionGroup.Group2)
                {
                    if(CanCollide)
                        lifeTime += (float)World.Time.ElapsedTime.Milliseconds;

                    if (lifeTime > boom_time_current)
                    {
                        SortZoneResponse.timeToEndSort += 1;
                        World.RemoveAllBobble(false);
                    }
                    float t = lifeTime / boom_time_current;


                    Color tmpX = Color.Magenta; ;
                    switch(info.color)
                    {
                        case BodyColor.Red: tmpX = BobbleDraw.BODY_RED;
                            break;
                        case BodyColor.Green: tmpX = BobbleDraw.BODY_GREEN;
                            break;
                        case BodyColor.Blue: tmpX = BobbleDraw.BODY_BLUE;
                            break;
                        

                    }

                    Color tmp = ColorUtil.Lerp(tmpX, ColorUtil.AcaOrange, t * t);
                    (Drawer as BobbleDraw).SetColor(new Color(tmp.R,tmp.G,tmp.B),true);
                }
               

                else if (Category == CollisionGroup.Group3)
                {
                    if (World.gameArea.Contains(Collider.Position.X, Collider.Position.Y))
                    {
                        this.Kill();
                    }
                }
                
               
            }




        }
        public void BobbleInZone()
        {
            Category = Entities.CollisionGroup.Group3;
            Mask = Entities.CollisionGroup.Group3 | CollisionGroup.Environment;
            (Drawer as BobbleDraw).SetBodyColor(info.color, 128);
        }
        public void ChangeBobble(BobbleInfo info)
        {
            this.info = info;

            BobbleDraw draw = (this.Drawer as BobbleDraw);
            draw.SetInfo(this.info);
            
            draw.Init();
            draw.SetInfo(this.info);

            draw.SetBodyColor(this.info.color);
           
        }
    }
}
