﻿using SFML.Graphics;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;


namespace BobbleCrowd.Components
{
    class BobbleDraw : ADrawer
    {
        AnimationSprite body;
        AnimationSprite eye;
        AnimationSprite mouth;


        public readonly static Color BODY_RED = new Color(255, 255, 0);
        public readonly static Color BODY_GREEN = new Color(0, 255, 255);
        public readonly static Color BODY_BLUE = new Color(255, 0, 255);


        private BobbleInfo info;

        public void SetInfo(BobbleInfo newInfo)
        {
            this.info = newInfo;
        }

        public void SetScale(float scale)
        {
            this.scale = scale;
            body.scale = scale;
            eye.scale = scale;
            mouth.scale = scale;

            mouth.SetScale(scale);
            eye.SetScale(scale);
            body.SetScale(scale);
        }



        private float scale;
        public BobbleDraw(float scale, BobbleInfo info)
        {
            this.scale = scale;
            this.info = info;
        }
        public override void Update()
        {
            body.Update(Owner.World.Time);
            body.position = Owner.Collider.Position;
            body.scale = scale;

            LinearSlowDownMover thinker = Owner.Thinker.GetComponentOfType<LinearSlowDownMover>();
            if(thinker!= null)
            {
                body.scale = (scale / thinker.startSpeed)*thinker.speed;
                //body.rotation = MathUtil.ToDegree(Vec2f.RotationFrom(thinker.direction))+90;
            }

            if (!info.mouth.Equals(MouthType.Count))
            {
                mouth.Update(Owner.World.Time);
                mouth.position = Owner.Collider.Position;
                mouth.scale = scale;
            }
            if (!info.mouth.Equals(MouthType.Count))
            {
                eye.Update(Owner.World.Time);
                eye.position = Owner.Collider.Position;
                eye.scale = scale;
            }

        }


        public override void Draw(SFML.Graphics.RenderTarget target)
        {
            body.Draw(target);
            if (!info.mouth.Equals(MouthType.Count))
                mouth.Draw(target);
            if (!info.eye.Equals(EyeType.Count))
                eye.Draw(target);
            //Owner.Collider.DebugDraw(target, Color.Red);
        }

        public override void Init()
        {
            body = new AnimationSprite("bobble/body/bodySprite.png", 100,scale);
            body.position = Owner.Collider.Position;
            body.scale = scale;
            body.Init();
            SetBodyColor(info.color);

            if (!info.mouth.Equals(MouthType.Count))
            {
                mouth = new AnimationSprite("bobble/mouth/mouthes.png", (int)info.mouth, (int)MouthType.Count, 100, scale);
                mouth.position = Owner.Collider.Position;
                mouth.scale = scale;
                mouth.Init();
            }
            if (!info.eye.Equals(EyeType.Count))
            {
                eye = new AnimationSprite("bobble/eye/eyes.png", (int)info.eye, (int)EyeType.Count, 100, scale);
                eye.position = Owner.Collider.Position;
                eye.scale = scale;
                eye.Init();
            }
            
        }

        public void SetBodyColor(BodyColor color)
        {
            switch (color)
            {
                case BodyColor.Red: body.SetColor(BODY_RED);
                    break;
                case BodyColor.Green: body.SetColor(BODY_GREEN);
                    break;
                case BodyColor.Blue: body.SetColor(BODY_BLUE);
                    break;

            }
        }
        public Color GetBodyColor()
        {
            return new Color(body.color);
        }
        public void SetBodyColor(BodyColor color, float alpha)
        {
            switch (color)
            {
                case BodyColor.Red: body.SetColor(new Color(BODY_RED.R, BODY_RED.G, BODY_RED.B, (byte)alpha));
                    break;
                case BodyColor.Green: body.SetColor(new Color(BODY_GREEN.R, BODY_GREEN.G, BODY_GREEN.B, (byte)alpha));
                    break;
                case BodyColor.Blue: body.SetColor(new Color(BODY_BLUE.R, BODY_BLUE.G, BODY_BLUE.B, (byte)alpha));
                    break;

            }
        }

        public void SetColor(Color color,bool onlyMouth)
        {
            if(!onlyMouth)
            {
                eye.SetColor(color);
                mouth.SetColor(color);
            }
                body.SetColor(color);
        }
    }
}
