﻿using BobbleCrowd.Entities;
using BobbleCrowd.World;
using SFML.Graphics;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BobbleCrowd.Components
{
    public abstract class AComponent
    {
        public Entity Owner
        {
            get;
            set;
        }

        public bool IsActive
        {
            get;
            set;
        }

        public abstract void Init();

        public abstract void Update();

        public virtual U GetComponentOfType<U>() where U : AComponent
        {
            if (this is U)
                return this as U;

            return null;
        }
    }

    //___________________DRAWER______________________________
    public abstract class ADrawer : AComponent
    {
        public bool toBeDrawn = true;

        public static ADrawer GetEmpty(bool debugDraw) { return new EmptyDrawer(debugDraw); }

        public abstract override void Update();

        public abstract void Draw(RenderTarget target);

        public abstract override void Init();

    }

    public sealed class EmptyDrawer : ADrawer
    {
        public bool debugDraw;

        public EmptyDrawer(bool debugDraw)
        {
            this.debugDraw = debugDraw;
        }

        public override void Draw(RenderTarget target)
        {

            if(debugDraw)
                Owner.Collider.DebugDraw(target, new Color(255, 0, 0, 127));
        }

        public override void Update()
        {

        }

        public override void Init()
        {

        }
    }


    //___________________THINKER______________________________
    public abstract class AThinker : AComponent
    {
        public static AThinker GetEmpty() { return new EmptyThinker(); }
        public abstract override void Update();

        public abstract override void Init();
    }

    public sealed class EmptyThinker : AThinker
    {

        public EmptyThinker()
        {

        }

        public override void Update()
        {

        }

        public override void Init()
        {

        }
    }


    //___________________COLLISION RESPONSE____________________
    public abstract class ACollisionResponse : AComponent
    {
        public static ACollisionResponse GetEmpty() { return new EmptyResponse(); }

        public abstract void OnCollision(Entity collidedEntity, IntersectionData data, bool firstCalled);

        public abstract override void Update();

        public abstract override void Init();
    }

    public sealed class EmptyResponse : ACollisionResponse
    {
        public EmptyResponse()
        {
        }


        public override void OnCollision(Entity collidedEntity, IntersectionData data, bool firstCalled)
        {

        }

        public override void Update()
        {

        }

        public override void Init()
        {

        }

    }


}
