﻿using BobbleCrowd.Entities;
using SFML.Graphics;
using SFML.Window;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BobbleCrowd.Components
{
    public sealed class PushResponse : ACollisionResponse
    {
        /// <summary> The weight of this Response, it decides how the objects will be separated. Only positive values. </summary>
        public float Weight
        {
            get;
            set;
        }

        /// <summary> Indicates if this component can be pushed, or if it only pushes. </summary>
        public bool IsPushable
        {
            get;
            set;
        }

        public PushResponse(bool pushable, float weight)
        {
            this.Weight = weight;

            this.IsPushable = pushable;

            this.IsActive = false;
        }

        public override void OnCollision(Entity collidedEntity, IntersectionData data, bool firstCalled)
        {

            if (this.IsPushable)
            {
                PushResponse otherResponse = collidedEntity.Response.GetComponentOfType<PushResponse>();

                if (otherResponse != null)
                {
                    Vector2f separation = data.direction * data.depth * (firstCalled ? -1.0f : 1.0f);

               

                    if (otherResponse.IsPushable)
                        Owner.Collider.Position += (separation * (otherResponse.Weight / (Weight + otherResponse.Weight)));
                    
                    else
                        Owner.Collider.Position += (separation);

                    /*
                    float radius = Owner.Collider.Radius;
                    FloatRect rect = Owner.World.gameArea;
                    Vector2f newPos = this.Owner.Collider.Position;


                    if (newPos.X - radius <= rect.Left)
                    {
                        float leftOverlap = (newPos.X - radius) - rect.Left;

                        if (leftOverlap < 0)
                            this.Owner.Collider.Move(-leftOverlap, 0);
                    }

                    else if (newPos.X + radius >= rect.Left + rect.Height)
                    {
                        float rightOverlap = (newPos.X + radius) - rect.Left + rect.Height;

                        if (rightOverlap > 0)
                            this.Owner.Collider.Move(rightOverlap, 0);
                    }
                      */

                }
            }
      
        }

        public override void Update()
        {
            
        }

        public override void Init()
        {
          
        }

    }
}
