﻿using SFML.Window;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BobbleCrowd.Components
{
    public class CircleMover : AThinker
    {
        public Vector2f center;
        public float radius;
        
        private double fullCircleTime;
        private double radStart;
        private double currentRad;
        private double updateRadChange;

        private bool clockWise;

        private Action DirectionUpdate;

        public CircleMover(Vector2f center, double radStart, float radius, double fullCircleTime, bool clockWise)
        {
            this.center = center;
            this.radius = radius;
            this.fullCircleTime = fullCircleTime;
            this.radStart = radStart;
            this.clockWise = clockWise;
            this.updateRadChange = (Math.PI * 2.0) / fullCircleTime;
            this.currentRad = radStart;

            if (clockWise)
                DirectionUpdate = ClockwiseUpdate;

            else
                DirectionUpdate = AntiClockwiseUpdate;
        }

        public CircleMover(Vector2f center, Vector2f directionStart, float radius, double fullCircleTime, bool clockWise)
            : this(center, Vec2f.RotationFrom(directionStart), radius, fullCircleTime, clockWise)
        {

        }

        private void ClockwiseUpdate()
        {
            Owner.Collider.Position = center + new Vector2f((float)Math.Cos(currentRad), (float)Math.Sin(currentRad)) * radius;
        }

        private void AntiClockwiseUpdate()
        {
            Owner.Collider.Position = center + new Vector2f((float)Math.Cos(currentRad), -(float)Math.Sin(currentRad)) * radius;
        }

        public override void Update()
        {
            currentRad += updateRadChange * Owner.World.Time.ElapsedTime.TotalSeconds;

            DirectionUpdate();
        }

        public override void Init()
        {
            Owner.Collider.Position = center + new Vector2f((float)Math.Cos((double)radStart), (float)Math.Sin((double)radStart)) * radius;
        }

    }
}
