﻿using SFML.Window;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BobbleCrowd.Components
{
    public class RandomMover : AThinker
    {
        float speed;
        float directionChangeRange;

        float targetRotation;
        float oldRotation;

        double changeDirectionCooldown;
        double maxCooldown;
        double chanceNotMoving;

        private float startRotation;

        bool notMoving = false;

        public RandomMover(float speed, double changeDirectionCooldown, float directionChangeRange, double chanceNotMoving, float startRotation = 0.0f)
        {
            this.speed = speed;
            this.changeDirectionCooldown = changeDirectionCooldown;
            this.maxCooldown = changeDirectionCooldown;
            this.directionChangeRange = directionChangeRange;
            this.chanceNotMoving = chanceNotMoving;

            this.startRotation = startRotation;
        }

        public override void Update()
        {
            changeDirectionCooldown -= Owner.World.Time.ElapsedTime.TotalSeconds;

            if (changeDirectionCooldown <= 0)
            {
                notMoving = false;
                changeDirectionCooldown = maxCooldown;
                float oldRot = Owner.Collider.Rotation;
                float min = oldRot - directionChangeRange;
                float max = oldRot + directionChangeRange;
                targetRotation = MathUtil.Rand.NextFloat(min, max);
                oldRotation = Owner.Collider.Rotation;

                if (MathUtil.Rand.NextDouble() < chanceNotMoving)
                    notMoving = true;
            }

            if (!notMoving)
            {
                float t = 1.0f - (float)(changeDirectionCooldown / maxCooldown);
                Owner.Collider.Rotation = (1.0f - t) * oldRotation + t * targetRotation;

                Vector2f newPos = Owner.Collider.Position + Owner.Collider.Direction * speed;
                float radius = Owner.Collider.Radius;

                if (newPos.X - radius <= Owner.World.gameArea.Left || newPos.X + radius >= Owner.World.gameArea.Left+Owner.World.gameArea.Width|| newPos.Y - radius <= Owner.World.gameArea.Top || newPos.Y + radius >= Owner.World.gameArea.Top+Owner.World.gameArea.Height)
                {
                    oldRotation = oldRotation - MathUtil.PI;
                    targetRotation = targetRotation - MathUtil.PI;
                }

                else
                    Owner.Collider.Position = newPos;
            }
                
        }

        public override void Init()
        {
            oldRotation = Owner.Collider.Rotation;
            targetRotation = startRotation;
        }

    }
}
