﻿using SFML.Graphics;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BobbleCrowd.Components
{
    public class TargetBobbleThinker : AThinker
    {
        private const double SPEED_UP_FACTOR            = 0.1; //percent speed up
        private const double START_TARGET_CHANGE_TIME   = 5.0;
        private const double INTERVALL_CHANGE_TIME      = 5.0;


        private List<Vector3i> usedInfoIndices = new List<Vector3i>();
        private int[, ,] usedInfos = new int[(int)BodyColor.Count, (int)MouthType.Count, (int)EyeType.Count];

        private double changeTargetTimeMax  = START_TARGET_CHANGE_TIME;
        private double changeTargetTime     = START_TARGET_CHANGE_TIME;

        private double currentIntervallRunning      = INTERVALL_CHANGE_TIME;
        private double intervallTimeMax             = INTERVALL_CHANGE_TIME;

        private bool setTransparant         = false;
        private int difficulty = 0;

        public TargetBobbleThinker()
        {

        }

        public override void Update()
        {
            double runDelta = Owner.World.Time.ElapsedTime.TotalSeconds;

            currentIntervallRunning -= runDelta;
            changeTargetTime -= runDelta;

            if (changeTargetTime <= 0)
            {
                changeTargetTime = changeTargetTimeMax;

                ChangeTarget();
            }

            if (currentIntervallRunning <= 0)
            {
                currentIntervallRunning = intervallTimeMax;
                SpeedUp();
            }

            //scale target bobble:
            float changeTargetT = (float)Math.Sqrt(Math.Sin((1.0 - changeTargetTime / changeTargetTimeMax) * Math.PI));
            (Owner.Drawer as BobbleDraw).SetScale(changeTargetT * 0.7f);

            if(changeTargetT<0.25)
                if(Owner.World.mode is BobbleMixMode)
                    (Owner.World.mode as BobbleMixMode).menu.targetBobbleChanged();


            if (usedInfoIndices.Count == 0)
            {
                if (!setTransparant)
                {

                    (Owner.Drawer as BobbleDraw).SetBodyColor((Owner as Bobble).info.color, 127);
                    setTransparant = true;
                }
                   
            }

            else
            {
                if (setTransparant)
                {
                    (Owner.Drawer as BobbleDraw).SetBodyColor((Owner as Bobble).info.color,255);
                    setTransparant = false;
                }
                    
            }

            Vector3i indices = (Owner as Bobble).info.GetVector();
            if (Owner.World.mode is BobbleMixMode)
                (Owner.World.mode as BobbleMixMode).menu.SetNumberOfCurrent(usedInfos[indices.x, indices.y, indices.z]);
        }

        private void SpeedUp()
        {
            difficulty++;

            if (difficulty % 3 == 0)
            {
                if (Owner.World.mode is BobbleMixMode)
                {
                     (Owner.World.mode as BobbleMixMode).respawnTimeMax *= (1.0 - SPEED_UP_FACTOR);
                     (Owner.World.mode as BobbleMixMode).numSpawns++;
                }
                   
            }

        }

        public override void Init()
        {
            ChangeTarget();
        }

        private void ChangeTarget()
        {
            
            if (usedInfoIndices.Count > 0)
            {
                int indice = MathUtil.Rand.Next(0, usedInfoIndices.Count);

                Vector3i indice3d = usedInfoIndices[indice];

                BobbleInfo info = new BobbleInfo(indice3d);

                (Owner as Bobble).ChangeBobble(info);
                
                
            }



        }

        public void AddInfo(BobbleInfo info)
        {
            Vector3i indices = info.GetVector();

            usedInfos[indices.x, indices.y, indices.z]++;

            usedInfoIndices.Add(indices);
        }

        public void RemoveInfo(BobbleInfo info)
        {
            Vector3i indices = info.GetVector();
            usedInfos[indices.x, indices.y, indices.z]--;
            usedInfoIndices.Remove(indices);
        }


        //TODO: re-enable
        public void TestChange()
        {
            //aktuelle kombination nicht vorhanden:
          //  if (!usedInfoIndices.Contains((Owner as Bobble).info.GetVector()))
          //  {
               // ChangeTarget();
           // }
        }
    }
}
