﻿using SFML.Graphics;
using SFML.Window;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BobbleCrowd
{
    public class CircleCollider : ACollider
    {
        public CircleCollider(float radius)
            : base(ECollidable.Circle)
        {

            this.radius = radius;
            this.scale = 1.0f;

            CalculateBoundingRectangle();

            this.Position = Vec2f.Zero;
        }

        public override IntersectionData Intersects(ACollider other)
        {
            if (other.CollidableType == ECollidable.Circle)
                return SatCircle(other as CircleCollider);

            else if (other.CollidableType == ECollidable.Convex)
                return SatConvex(other as ConvexCollider);

         //   else if (other.CollidableType == ECollidable.Concave)
        //        return (other as ConcaveCollider).SatCircle(this, true);

            else
                return new IntersectionData();
        }

        public IntersectionData SatCircle(CircleCollider other)
        {
            CircleCollider otherTmp = (other as CircleCollider);

            float r = radius + otherTmp.radius;

            Vector2f direction = otherTmp.position - position;

            float dist = Vec2f.Length(direction);

            if (dist < r)
                return new IntersectionData(r - dist, Vec2f.Normalized(direction, dist));

            else
                return new IntersectionData();
        }

        public IntersectionData SatConvex(ConvexCollider other)
        {
            IntersectionData tmpData = other.SatCircle(this);

            if (tmpData.Intersects)
                tmpData.direction *= -1;

            return tmpData;
        }


        public override void Rotate(float deltaRad)
        {
            rotation += deltaRad;
            direction = Vec2f.DirectionFrom(rotation);
        }



        public override void Move(float deltaX, float deltaY)
        {
            position.X += deltaX;
            position.Y += deltaY;

            MoveBoundingRectangle();
        }

        public override void ScaleUp(float scaleFactor)
        {
            if (scaleFactor > 1.0f || (scaleFactor < 1.0f && scale > MathUtil.FLOAT_ZERO_TOLERANCE))
            {
                radius *= scaleFactor;
                scale *= scaleFactor;
            }
        }

        public override bool IntersectsPoint(Vector2f point)
        {
            return Vec2f.EuclidianDistanceSq(point, position) <= radius * radius;
        }

        public override ACollider Copy()
        {
            return new CircleCollider(this.radius);
        }


        public override void DebugDraw(RenderTarget target, Color color)
        {
            CircleShape circle = new CircleShape(this.radius);
            circle.Position = this.position;
            circle.Origin = new Vector2f(this.radius, this.radius);
            circle.FillColor = color;

            target.Draw(circle);
        }
    }
}
