﻿using SFML.Graphics;
using SFML.Window;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Project_Darkness
{
    public class ConcaveCollider : ACollider
    {

        private List<ConvexCollider> subPolys;

        public int VertexCount
        {
            get;
            private set;
        }

        public ConcaveCollider(List<ConvexCollider> info, Vector2f center, int vertexCount)
            : base(ECollidable.Concave)
        {
            subPolys = info;

            this.position = center;
            VertexCount = vertexCount;

            computerRadius();
            CalculateBoundingRectangle();
        }



        public override IntersectionData Intersects(ACollider other)
        {
            if (other.CollidableType == ECollidable.Circle)
                return SatCircle(other as CircleCollider, false);

            else if (other.CollidableType == ECollidable.Convex)
                return SatConvex(other as ConvexCollider, false);


            else if (other.CollidableType == ECollidable.Concave)
                return SatConcave(other as ConcaveCollider);

            else
                return new IntersectionData();
        }

        public IntersectionData SatCircle(CircleCollider other, bool swapDir)
        {
            if (this.IntersectsBounding(other))
            {
                IntersectionData totalData = new IntersectionData();
                totalData.depth = new List<float>();
                totalData.direction = new List<Vector2f>();

                for (int i = 0; i < subPolys.Count; i++)
                {
                    //test if the given SubPolygon intersects with the other given Polygon:
                    IntersectionData data = subPolys[i].SatCircle(other);

                    //if they intersect, store the data inside totalData.
                    if (data.Intersects)
                    {
                        //add the new values for the given intersection:
                        totalData.depth += data.depth;

                        if (swapDir)
                            totalData.direction -= data.direction;
                        else
                            totalData.direction += data.direction;
                    }
                }

                return totalData;
            }
            else
                return new IntersectionData();
        }

        public IntersectionData SatConcave(ConcaveCollider other)
        {
            if (this.IntersectsBounding(other))
            {

                IntersectionData totalData = new IntersectionData();
                totalData.depth = new List<float>();
                totalData.direction = new List<Vector2f>();


                for (int i = 0; i < this.subPolys.Count; i++)
                {
                    IntersectionData tmpData = this.SatConvex(other.subPolys[i], false);

                    if (tmpData.Intersects)
                    {
                      
                            totalData.depth += tmpData.depth;
                            totalData.direction += tmpData.direction;
                        
                    }
                }

                return totalData;
            }
            else
                return new IntersectionData();

        }

        //TODO: maybe add maximum intersections (e.g. 2)
        public IntersectionData SatConvex(ConvexCollider other, bool swapDir)
        {
            if (this.IntersectsBounding(other))
            {
                //create PolyColData where to store the intersection information.
                IntersectionData totalData = new IntersectionData();
                totalData.depth = new List<float>();
                totalData.direction = new List<Vector2f>();

                //go throw all ConvexPoly's this ConcavePoly is made of:
                for (int i = 0; i < subPolys.Count; i++)
                {
                    //test if the given SubPolygon intersects with the other given Polygon:
                    IntersectionData data = subPolys[i].SatConvex(other);

                    //if they intersect, store the data inside totalData.
                    if (data.Intersects)
                    {
                        //add the new values for the given intersection:
                        totalData.depth.Add(data.depth[0]);

                        if (swapDir)
                            totalData.direction -=(-data.direction[0]);
                        else
                            totalData.direction.Add(data.direction[0]);
                    }

                }
                return totalData;

            }
            else
                return new IntersectionData();
        }


        public override void Rotate(float deltaRad)
        {
            for (int i = 0; i < subPolys.Count; i++)
                subPolys[i].Rotate(deltaRad, position);

            rotation += deltaRad;
            direction = Vec2f.DirectionFrom(rotation);

            ClampRotation();
        }

        public override void Move(float deltaX, float deltaY)
        {
            for (int i = 0; i < subPolys.Count; i++)
                subPolys[i].Move(deltaX, deltaY);

            position.X += deltaX;
            position.Y += deltaY;

            MoveBoundingRectangle();
        }

        public override void ScaleUp(float scaleFactor)
        {
            for (int i = 0; i < subPolys.Count; i++)
            {
                subPolys[i].ScaleUp(scaleFactor, position);
            }

            radius *= scaleFactor;
            scale *= scaleFactor;
        }


        private void computerRadius()
        {
            float maxRadius = float.NegativeInfinity;

            foreach (ConvexCollider tmpCol in subPolys)
            {
                for (int i = 0; i < tmpCol.VertexCount; i++)
                {
                    float tmpRadius = Vec2f.EuclidianDistance(position, tmpCol[i]);

                    if (tmpRadius > maxRadius)
                        maxRadius = tmpRadius;
                }
            }

            radius = maxRadius;
        }

        public override ACollider Copy()
        {
            List<ConvexCollider> sub = new List<ConvexCollider>();

            for (int i = 0; i < this.subPolys.Count; i++)
                sub.Add((this.subPolys[i].Copy() as ConvexCollider));

            return new ConcaveCollider(sub, this.Position, VertexCount);
        }

        public override bool IntersectsPoint(Vector2f point)
        {
            for (int i = 0; i < subPolys.Count; i++)
            {
                if (subPolys[i].IntersectsPoint(point))
                    return true;
            }

            return false;
        }

        public override void DebugDraw(RenderTarget target, Color color)
        {
            for (int i = 0; i < subPolys.Count; i++)
                subPolys[i].DebugDraw(target, color);
            
        }
    }
}
