﻿using SFML.Graphics;
using SFML.Window;
using BobbleCrowd.Template;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Diagnostics;
using BobbleCrowd.GameStates;
using BobbleCrowd.Message;

//TODO: add savegames anywhere.
namespace BobbleCrowd
{
    public sealed class Game : AbstractGame, IDisposable
    {
        public static ContentManager Content
        {
            get;
            private set;
        }

        private static Queue<AMessage> messages;
        private static bool isEmptyMessage = true;
     
        private static AMessage currentMessage;

        public static void AddMessage(AMessage message)
        {
            //TODO: how to pass in the correct params
            message.Init(null, null);
            messages.Enqueue(message);
        }

        private AGameState gameState;

        public Game()
        {
            Content = new ContentManager();

            messages = new Queue<AMessage>();
            currentMessage = EmptyMessage.Instance;

            InputManager.CreateInputDevices(this, false);

            gameState = new MainMenu();
            gameState.setParent(this);
            gameState.Init(null);


            Image i = new Image("Content/icon.png");
            window.SetIcon(128, 128, i.Pixels);
        }



        protected override void Update(GameTime gameTime)
        {
          //  window.SetTitle("" + (1.0 /gameTime.ElapsedTime.TotalSeconds));
            InputManager.Update(gameTime);
   
            if(!currentMessage.Pauses)
                gameState.Update();

            //handle messages:

            //the current message has finished and will be "destroyed" now
            if (currentMessage.HasFinished)
            {
                //there are still messages in the queue, take from it:
                if (messages.Count > 0)
                {
                    currentMessage = messages.Dequeue();
                    isEmptyMessage = false;
                }

                //no more messages, take the empty message now:
                else if (!isEmptyMessage)
                {
                    currentMessage.ShutDown(gameTime, window);
                    currentMessage = EmptyMessage.Instance;
                    isEmptyMessage = true;
                }

            }


            currentMessage.Update(gameTime, window);
        }

        //TODO: maybe remove draw methods and add Renderer -> VertexArray with specific Texture -> faster
        protected override void Draw(GameTime gameTime, RenderWindow window)
        {
            gameState.Draw();

            currentMessage.Draw(gameTime, window);

        }

        public int mouseWheel()
        {
            return this.wheelDelta;
        }

        /// <summary>
        /// Shuts down the old gamestate, then starts up the new gamestate. 
        /// </summary>
        /// <param name="newGameState">The new gamestate, closes the application when passing null.</param>
        public void SetGameState(AGameState newGameState)
        {
            this.gameState.ShutDown();

            if (newGameState == null)
            {
                this.gameState = new EmptyGameState();
                this.window.Close();     
                return;
            }

            else
            {
                AGameState lastGameState = this.gameState;

                this.gameState = newGameState;

                this.gameState.setParent(this);

                this.gameState.Init(lastGameState);
            }
        }


        public void Dispose()
        {
            Content.Dispose();
        }

    }
}
