﻿using BobbleCrowd.World;
using SFML.Graphics;
using SFML.Window;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BobbleCrowd.GameStates
{
    public class HighscoreTable : AGameState
    {
        private readonly static char[] keyCharMap = {    'a', 'b', 'c', 'd', 'e', 'f', 
                                                'g', 'h', 'i', 'j', 'k', 'l', 
                                                'm', 'n', 'o', 'p', 'q', 'r', 
                                                's', 't', 'u', 'v', 'w', 'x', 
                                                'y', 'z',
                                                '0','1','2','3','4','5','6','7','8','9' };

        private const String TABLE_NAME_MIX = "Content/other/mix.bob";
        private const String TABLE_NAME_SORT = "Content/other/sort.bob";
        private const String MESSAGE_WIN = "Congratulations, you have enough points! Please type in your name!";
        private const String MESSAGE_LOSE = "Sorry, you don't have enough points. Try harder!";
        private const String TITLE_MIX = "Mix-Mode";
        private const String TITLE_SORT = "Sort-Mode";
        private const String CONTINUE_STRING = "Escape to continue...";

        private const float BORDER_DISTANCE = 16.0f;

        private readonly Vector2f mixTablePosStart = new Vector2f(BORDER_DISTANCE, 50.0f + BORDER_DISTANCE);
        private readonly Vector2f titleMixPos = new Vector2f(BORDER_DISTANCE, BORDER_DISTANCE);

        private readonly Vector2f sortTablePosStart = new Vector2f(Settings.WindowSize.X * 0.5f + BORDER_DISTANCE, 50.0f + BORDER_DISTANCE);
        private readonly Vector2f titleSortPos = new Vector2f(Settings.WindowSize.X * 0.5f + BORDER_DISTANCE, BORDER_DISTANCE);

        private readonly Vector2f distanceEntries = new Vector2f(0, 50.0f);
        private readonly Vector2f messagePos = new Vector2f(BORDER_DISTANCE, Settings.WindowSize.Y * 0.8f + BORDER_DISTANCE);
        private readonly Vector2f writeBoxPos = new Vector2f(BORDER_DISTANCE, Settings.WindowSize.Y * 0.9f + BORDER_DISTANCE);



        private List<HighscoreEntry> mixEntries = new List<HighscoreEntry>();
        private List<HighscoreEntry> sortEntries = new List<HighscoreEntry>();
        private Text titleMix;
        private Text titleSort;


        private List<char> currentChars = new List<char>();
        private bool enableWriting = false;

        private ModeType type = ModeType.count;
        private Text message;
        private Text writeBox;
        private int newPoints = -1;



        public HighscoreTable()
        {

        }

        public override void Init(AGameState lastGameState)
        {
            LoadTables();

            message = new Text(CONTINUE_STRING, Game.Content.Load<Font>(Strings.FONT_NAME));
            message.Position = messagePos;

            titleMix = new Text(TITLE_MIX, Game.Content.Load<Font>(Strings.FONT_NAME));
            titleSort = new Text(TITLE_SORT, Game.Content.Load<Font>(Strings.FONT_NAME));

            titleMix.Style = Text.Styles.Bold | Text.Styles.Underlined;
            titleSort.Style = Text.Styles.Bold | Text.Styles.Underlined;

            titleMix.Position = titleMixPos;
            titleSort.Position = titleSortPos;

            if (lastGameState is InGame)
            {
                InGame inGame = (lastGameState as InGame);

                GameWorld world = inGame.GetDrawGroup<GameWorld>(Strings.TEXTURE_DRAWER_GAMEWORLD);

                if (world != null)
                {
                    if (world.mode is BobbleSortMode)
                    {
                        type = ModeType.BOBBLE_SORT;
                        BobbleSortMode mode = (world.mode as BobbleSortMode);

                        if (mode.menu.points > sortEntries[sortEntries.Count - 1].points)
                        {
                            newPoints = mode.menu.points;
                            message.DisplayedString = MESSAGE_WIN;
                            enableWriting = true;
                            writeBox = new Text("", Game.Content.Load<Font>(Strings.FONT_NAME));
                         //   writeBox.Color = Color.Red;
                        }
                        else
                        {
                            message.DisplayedString = MESSAGE_LOSE + "\n" + CONTINUE_STRING;
                        }
                    }

                    else if (world.mode is BobbleMixMode)
                    {
                        BobbleMixMode mode = (world.mode as BobbleMixMode);
                        type = ModeType.BOBBLE_MIX;

                        if (mode.menu.points > mixEntries[sortEntries.Count - 1].points)
                        {
                            newPoints = mode.menu.points;
                            message.DisplayedString = MESSAGE_WIN;
                            enableWriting = true;
                            writeBox = new Text("", Game.Content.Load<Font>(Strings.FONT_NAME));
                        }

                        else
                        {
                            message.DisplayedString = MESSAGE_LOSE + "\n" + CONTINUE_STRING;
                        }
                    }

                    else
                        message.DisplayedString = CONTINUE_STRING;
                }
            }

            if (writeBox != null)
            {
                writeBox.Position = writeBoxPos;
            }
        }

        private void LoadTables()
        {
            String[] mixStringEntries = File.ReadAllLines(TABLE_NAME_MIX);
            String[] sortStringEntries = File.ReadAllLines(TABLE_NAME_SORT);
            char[] splits = { ',' };

            for (int i = 0; i < 9; i++)
            {

                String[] entry = mixStringEntries[i].Split(splits, StringSplitOptions.RemoveEmptyEntries);
                mixEntries.Add(new HighscoreEntry(entry[1], int.Parse(entry[0])));
            }

            for (int i = 0; i < 9; i++)
            {

                String[] entry = sortStringEntries[i].Split(splits, StringSplitOptions.RemoveEmptyEntries);
                sortEntries.Add(new HighscoreEntry(entry[1], int.Parse(entry[0])));

            }

            SetPositions(mixEntries, ModeType.BOBBLE_MIX);
            SetPositions(sortEntries, ModeType.BOBBLE_SORT);
        }

        public override void ShutDown()
        {
            
        }

        public override void Update()
        {
            if (enableWriting)
            {
                int count = currentChars.Count;

                char currentTypeIn = GetClickedChar();

                if (currentTypeIn != '-')
                {
                    if (KInput.IsPressed(SFML.Window.Keyboard.Key.LShift) || KInput.IsPressed(SFML.Window.Keyboard.Key.RShift))
                        currentTypeIn = char.ToUpper(currentTypeIn);

                    if(currentChars.Count < 20)
                        currentChars.Add(currentTypeIn);
                }
                   

                if (KInput.IsClicked(SFML.Window.Keyboard.Key.Back) && KInput.IsPressed(SFML.Window.Keyboard.Key.LShift))
                    ClearAll();

                else if (KInput.IsClicked(SFML.Window.Keyboard.Key.Back) && currentChars.Count > 0)
                    currentChars.RemoveAt(currentChars.Count - 1);

                else if (KInput.IsClicked(SFML.Window.Keyboard.Key.Return))
                    FinishText();

                if (count != currentChars.Count)
                    CreateCurrent();
            }
            else
            {
                if (KInput.IsClicked(Keyboard.Key.Escape))
                    parent.SetGameState(new MainMenu());
            }

            
        }

        private void CreateCurrent()
        {
            writeBox.DisplayedString = String.Concat(currentChars);
        }

        private void SetPositions(List<HighscoreEntry> entries, ModeType type)
        {
            for (int i = 0; i < entries.Count; i++)
            {
                if (type == ModeType.BOBBLE_MIX)
                    entries[i].SetPositions(mixTablePosStart + distanceEntries * i);
                else
                    entries[i].SetPositions(sortTablePosStart + distanceEntries * i);
            }
        }

        private void FinishText()
        {
            enableWriting = false;
            HighscoreEntry newEntry = new HighscoreEntry(writeBox.DisplayedString, newPoints);

            if (type == ModeType.BOBBLE_MIX)
            {
                int index = GetInsertIndex(mixEntries, newPoints);
                mixEntries.Insert(index, newEntry);
                SetPositions(mixEntries, ModeType.BOBBLE_MIX);
                mixEntries.RemoveAt(mixEntries.Count - 1);
                Save(mixEntries, ModeType.BOBBLE_MIX);
            }


            else if (type == ModeType.BOBBLE_SORT)
            {
                int index = GetInsertIndex(sortEntries, newPoints);
                sortEntries.Insert(index, newEntry);
                SetPositions(sortEntries, ModeType.BOBBLE_SORT);
                sortEntries.RemoveAt(sortEntries.Count - 1);
                Save(sortEntries, ModeType.BOBBLE_SORT);
            }

            writeBox = null;

            message.DisplayedString = CONTINUE_STRING;
        }

        private void Save(List<HighscoreEntry> entries, ModeType type)
        {
            String[] arr = new String[entries.Count];

            for (int i = 0; i < entries.Count; i++)
            {
                arr[i] = entries[i].ToString();
            }

            if (type == ModeType.BOBBLE_MIX)
                File.WriteAllLines(TABLE_NAME_MIX, arr);

            else if (type == ModeType.BOBBLE_SORT)
                File.WriteAllLines(TABLE_NAME_SORT, arr);
        }

        private int GetInsertIndex(List<HighscoreEntry> entries, int point)
        {
            int curIndex = -1;
            for (int i = entries.Count - 1; i >= 0; i--)
            {
                if (entries[i].points < point)
                    curIndex = i;

                else
                    return curIndex;
            }

            return curIndex;
        }

        private void ClearAll()
        {
            currentChars.Clear();
        }

        public override void Draw()
        {
            parent.window.Clear();

            parent.window.Draw(titleMix);
            parent.window.Draw(titleSort);

            for (int i = 0; i < mixEntries.Count; i++)
            {
                mixEntries[i].Draw(parent.window);
            }

            for (int i = 0; i < sortEntries.Count; i++)
            {
                sortEntries[i].Draw(parent.window);
            }


            parent.window.Draw(message);

            if(writeBox != null)
                parent.window.Draw(writeBox);


        }

        private char GetClickedChar()
        {
            for (int i = 0; i < keyCharMap.Length; i++)
            {
                if (KInput.IsClicked((SFML.Window.Keyboard.Key)i))
                    return keyCharMap[i];
            }

            if (KInput.IsClicked(SFML.Window.Keyboard.Key.Space))
                return ' ';

            return '-';
        }
    }
}
