﻿using BobbleCrowd.Message;
using BobbleCrowd.World;
using SFML.Graphics;
using SFML.Window;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BobbleCrowd.GameStates
{
    public sealed class InGame : AGameState
    {
        private List<TextureDrawGroup> drawGroups;
        private ModeType type;

        public InGame(ModeType type)
        {
            this.type = type;
        }

        public T GetDrawGroup<T>(String name) where T : TextureDrawGroup
        {
            for (int i = 0; i < drawGroups.Count; i++)
            {
                if ((drawGroups[i] is T) && drawGroups[i].Name.Equals(name))
                    return drawGroups[i] as T;
            }

            return null;
        }
           

        public override void Init(AGameState lastGameState)
        {
            drawGroups = new List<TextureDrawGroup>();

            GameWorld world = new GameWorld();
            world.Create( parent.gameTime, this, this.type);
            drawGroups.Add(world);

            world.Init();

            if(world.mode is BobbleMixMode)
                (world.mode as BobbleMixMode).menu.world = world;

        }

        public override void ShutDown()
        {
            foreach (TextureDrawGroup group in drawGroups)
                group.Dispose();

            Game.Content.Dispose();
        }

        public override void Update()
        {
            foreach (TextureDrawGroup obj in drawGroups)
                obj.Update(parent.gameTime);

            if (KInput.IsClicked(Keyboard.Key.Escape))
                Game.AddMessage(new PauseMenu(parent));

        }

        public override void Draw()
        {
            parent.window.Clear();
            
            foreach (TextureDrawGroup obj in drawGroups)
            {
                obj.Draw(parent.gameTime);
                parent.window.Draw(new Sprite(obj.GetTexture()));
            }




        }

    }
}
