﻿using BobbleCrowd.Gui;
using SFML.Graphics;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BobbleCrowd.GameStates
{
    public class MainMenu : AGameState
    {
        private const float PERCENT_DISTANCE = 1.5f;

        private List<Button> buttons = new List<Button>();
        private String[] buttonStrings = {"Start", "Highscore", "Credits", "Exit" };


        public MainMenu()
        {

        }

        public override void Init(AGameState lastGameState)
        {
            for (int i = 0; i < buttonStrings.Length; i++)
            {
                buttons.Add(new Button(new SFML.Window.Vector2f(Settings.WindowSize.X * 0.5f, Settings.WindowSize.Y * 0.33f + i * 100.0f), buttonStrings[i]));
            }

            buttons[0].SetActions(OnClickStart, Button.OnHover, Button.OnEnter, Button.OnExit);
            buttons[1].SetActions(OnClickHighscore, Button.OnHover, Button.OnEnter, Button.OnExit);
            buttons[2].SetActions(OnClickCredits, Button.OnHover, Button.OnEnter, Button.OnExit);
            buttons[3].SetActions(OnClickExit, Button.OnHover,  Button.OnEnter, Button.OnExit);
        }

        public override void ShutDown()
        {
            MInput.CursorVisible = false;
        }

        public override void Update()
        {
            if(!MInput.CursorVisible)
                MInput.CursorVisible = true;

            foreach (Button b in buttons)
            {
                b.Update(parent.gameTime);
            }
        }

        public override void Draw()
        {
            parent.window.Clear();

            foreach (Button b in buttons)
            {
                b.Draw(parent.window);
            }
        }

        private void OnClickStart(Button button)
        {
            parent.SetGameState(new ModeSelection());
        }

        private void OnClickCredits(Button button)
        {
            parent.SetGameState(new Credits());
        }

        private void OnClickExit(Button button)
        {
            parent.window.Close();
            parent.SetGameState(new EmptyGameState());
        }

        private void OnClickHighscore(Button button)
        {
            parent.SetGameState(new HighscoreTable());
        }
    }
}
