﻿using BobbleCrowd.Gui;
using SFML.Graphics;
using SFML.Window;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BobbleCrowd.GameStates
{
    public class ModeSelection : AGameState
    {
        private String[] strings = {"Bobble-Mix", "Bobble-Sort", "Back" };

        private String sortText = "In this gamemode you have to sort the different Bobbles into their\nrelated zones, to avoid them from going mad and kill everything.";
        private String mixText = "In this gamemode you have to find as many certain Bobbles as fast\nas possible. But be careful, the Bobbles you have to find change\nperiodically!";

        private List<Button> buttons = new List<Button>();


        private Vector2f preViewPos = new Vector2f( Settings.WindowSize.X * 0.4f, 
                                                    Settings.WindowSize.Y * 0.1f);

        private Vector2f preViewTextPos = new Vector2f(Settings.WindowSize.X * 0.05f,
                                                        Settings.WindowSize.Y * 0.75f);

        private Sprite mixPreView;
        private Sprite sortPreView;
        private Sprite emptyPreView;

        private Sprite currentPreView;

        private Text preViewText;


        public ModeSelection()
        {

        }

        public override void Init(AGameState lastGameState)
        {
            preViewText = new Text("", Game.Content.Load<Font>(Strings.FONT_NAME));
            preViewText.Position = preViewTextPos;

            mixPreView = new Sprite(Game.Content.Load<Texture>(Strings.PREVIEW_MIX));
            mixPreView.Position = preViewPos;

            sortPreView = new Sprite(Game.Content.Load<Texture>(Strings.PREVIEW_SORT));
            sortPreView.Position = preViewPos;

            emptyPreView = new Sprite(Game.Content.Load<Texture>(Strings.PREVIEW_NONE));
            emptyPreView.Position = preViewPos;

            currentPreView = null;

            for (int i = 0; i < strings.Length; i++)
            {
                buttons.Add(new Button(new SFML.Window.Vector2f(Settings.WindowSize.X * 0.2f, Settings.WindowSize.Y * 0.175f + i * 100.0f), strings[i]));

            }

            buttons[0].SetActions(OnClickBobbleMix,     Button.OnHover, OnEnterMix,     OnExit);
            buttons[1].SetActions(OnClickBobbleSort,    Button.OnHover, OnEnterSort,    OnExit);
            buttons[2].SetActions(OnClickBack,          Button.OnHover, OnEnterNone, OnExit);
        }

        private void OnClickBack(Button obj)
        {
            parent.SetGameState(new MainMenu());
        }

        private void OnClickBobbleSort(Button obj)
        {
            parent.SetGameState(new InGame(World.ModeType.BOBBLE_SORT));
        }

        private void OnClickBobbleMix(Button obj)
        {
            parent.SetGameState(new InGame(World.ModeType.BOBBLE_MIX));
        }

        public override void ShutDown()
        {
            MInput.CursorVisible = false;
        }

        public override void Update()
        {
            if (!MInput.CursorVisible)
                MInput.CursorVisible = true;

            foreach (Button b in buttons)
            {
                b.Update(parent.gameTime);
            }

                if (KInput.IsClicked(Keyboard.Key.Escape)){
                  //  parent.window.Close();
                    parent.SetGameState(new MainMenu());
                }
                    
        }

        public override void Draw()
        {
            parent.window.Clear();

            foreach (Button b in buttons)
            {
                b.Draw(parent.window);
            }

            if (currentPreView != null)
                parent.window.Draw(currentPreView);

            parent.window.Draw(preViewText);
        }

        private void OnEnterMix(Button button)
        {
            Button.OnEnter(button);
            currentPreView = mixPreView;
            preViewText.DisplayedString = mixText;
        }

        private void OnEnterNone(Button button)
        {
            Button.OnEnter(button);
            currentPreView = emptyPreView;
            preViewText.DisplayedString = "";
        }

        private void OnEnterSort(Button button)
        {
            Button.OnEnter(button);
            currentPreView = sortPreView;
            preViewText.DisplayedString = sortText;
        }

        private void OnExit(Button button)
        {
            Button.OnExit(button);
            preViewText.DisplayedString = "";
            currentPreView = null;
        }
    }
}
