﻿using BobbleCrowd.GameStates;
using SFML.Graphics;
using SFML.Window;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BobbleCrowd.Gui
{
    public class Button
    {
        public Text text;
        private FloatRect rect;

        private bool contains = false;

        private Action<Button> onClick;
        private Action<Button> onHover;
        private Action<Button> onEnter;
        private Action<Button> onExit;

        public Button(Vector2f centerPos, String text)
        {
            this.text = new Text(text, Game.Content.Load<Font>(Strings.FONT_NAME));
            this.text.Color = Color.White;
            FloatRect bounds = this.text.GetLocalBounds();

            this.text.Position = centerPos;

            this.text.Origin = new Vector2f(bounds.Width * 0.5f, bounds.Height * 0.5f);

            this.rect = this.text.GetGlobalBounds();
        }

        public void SetActions(Action<Button> onClick, Action<Button> onHover, Action<Button> onEnter, Action<Button> onExit)
        {
            this.onClick = onClick;
            this.onHover = onHover;
            this.onEnter = onEnter;
            this.onExit = onExit;
        }

        public void Update(GameTime gameTime)
        {
            Vector2f mPos = MInput.WindowPos();

            if (!contains && rect.Contains(mPos.X, mPos.Y))
            {
                contains = true;
                onEnter(this);
            }
              
            else if (contains && !rect.Contains(mPos.X, mPos.Y))
            {
                contains = false;
                onExit(this);
            }

            else if (contains)
            {
                onHover(this);

                if (MInput.IsClicked(Mouse.Button.Left))
                {
                    onClick(this);
                }
            }
                

           
        }

        public void Draw(RenderTarget target)
        {
            target.Draw(text);
        }

        public bool Contains()
        {
            return contains;
        }



        public static void OnEnter(Button button)
        {
            button.text.Color = Color.Red;
        }

        public static void OnExit(Button button)
        {
            button.text.Color = Color.White;
            button.text.Scale = Vec2f.One;
        }

        public static void Nothing(Button button)
        {

        }

        public static void OnHover(Button button)
        {
            // button.text.Scale = Vec2f.One * (float)Math.Sin(parent.gameTime.TotalTime.TotalSeconds);
        }
    }
}
