﻿using SFML.Graphics;
using SFML.Window;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BobbleCrowd
{
    public class GInput
    {
        public enum EButton { A, B, X, Y, LB, RB, Back, Start, L3, R3, ButtonCount };

        private Input.GamepadKeyInfo[] infos;

        uint index;

        public GInput(uint index)
        {
            this.index = index;

            infos = new Input.GamepadKeyInfo[(int)EButton.ButtonCount];

            for (uint i = 0; i < infos.Length; i++)
                infos[i] = new Input.GamepadKeyInfo(index, i);
            
        }

        /// <summary>
        /// Requires SFML.Window.Joystick.Update() before this call.
        /// </summary>
        public void Update(GameTime gameTime)
        {
            for (uint i = 0; i < infos.Length; i++)
            {
                infos[i].Update(gameTime);
            }
        }

        public bool IsClicked(EButton button)
        {
            return infos[(int)button].IsClicked();
        }

        public bool IsPressed(EButton button)
        {
            return infos[(int)button].IsPressed();
        }

        public bool IsReleased(EButton button)
        {
            return infos[(int)button].IsReleased();
        }

        private Vector2f GetAxis(Joystick.Axis a, Joystick.Axis b)
        {
            return new Vector2f(Joystick.GetAxisPosition(index, a), Joystick.GetAxisPosition(index, b));
        }

        public Vector2f DPad()
        {
            return Vec2f.NegatedY(GetAxis(Joystick.Axis.PovY, Joystick.Axis.PovX));
        }

        public Vector2f RightPad()
        {
            return GetAxis(Joystick.Axis.U, Joystick.Axis.R);
        }

        public Vector2f LeftPad()
        {
            return GetAxis(Joystick.Axis.X, Joystick.Axis.Y);
        }

        /// <summary>
        /// Negative values for left bumper, positive values for right bumper.
        /// </summary>
        /// <returns></returns>
        public float Bumber()
        {
            return -Joystick.GetAxisPosition(index, Joystick.Axis.Z);
        }


    }
}
