﻿using BobbleCrowd.Input;
using SFML.Window;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

/// <summary>
/// Represents keyboard. Just call update() every tick once (at the beginning).
/// </summary>

namespace BobbleCrowd
{
    public static class KInput
    {
        private static KeyInfo[] keys;

        //TODO: add later needed keys:
        private static List<Keyboard.Key> usedKeys;

        public static void Init(List<Keyboard.Key> prefKeys = null)
        {
            keys = new KeyInfo[(int)Keyboard.Key.KeyCount];

            for (int i = 0; i < keys.Length; i++)
                keys[i] = new KeyboardKeyInfo((Keyboard.Key)i);


            if (prefKeys == null)
            {
                usedKeys = new List<Keyboard.Key>();

                for (int i = 0; i < (int)Keyboard.Key.KeyCount; i++)
                    usedKeys.Add((Keyboard.Key)i);     
            }
          
            else
                usedKeys = prefKeys;
        }

        public static void AddKey(Keyboard.Key key)
        {
            if(!usedKeys.Contains(key))
                usedKeys.Add(key);
        }

        public static void RemoveKey(Keyboard.Key key)
        {
            keys[(int)key].Reset();
            usedKeys.Remove(key);    
        }

        /// <summary>
        /// Checks every mouseButton and every used key (that were given within the constructor) if pressed.
        /// </summary>
        public static void Update(GameTime gameTime)
        {
            foreach (Keyboard.Key key in usedKeys)
                keys[(int)key].Update(gameTime);
        }

        /// <summary>
        /// If the given key is clicked (pressed right NOW and NOT pressed last tick).
        /// </summary>
        /// <param name="key">Which key to be checked.</param>
        /// <returns>True if key is clicked, false otherwise.</returns>
        public static bool IsClicked(Keyboard.Key key)
        {
            return keys[(int)key].IsClicked();
        }

        /// <summary>
        /// If the given key is pressed right NOW.
        /// </summary>
        /// <param name="key">Which key to be checked.</param>
        /// <returns>True if the given key is pressed, false otherwise.</returns>
        public static bool IsPressed(Keyboard.Key key)
        {
            return keys[(int)key].IsPressed();
        }

        /// <summary>
        /// Checks if the given key is released. (Not pressed right now and pressed last tick).
        /// </summary>
        /// <param name="key">Which key to be checked.</param>
        /// <returns></returns>
        public static bool IsReleased(Keyboard.Key key)
        {
            return keys[(int)key].IsReleased();
        }

        public static double GetPressTime(Keyboard.Key key)
        {
            return keys[(int)key].PressTime;
        }

        public static void Reset()
        {
            for (int i = 0; i < keys.Length; i++)
                keys[i].Reset();
        }

    }

}