﻿using SFML.Graphics;
using SFML.Window;
using System;

namespace BobbleCrowd
{

    /// <summary>
    /// Represents mouse. Just call update() every tick once (at the beginning).
    /// </summary>
    public static class MInput
    {
        private static Input.MouseKeyInfo[] infos;

        private static Vector2i oldMousePos;
        private static Vector2i currentMousePos;

        private static Game game;

        private static Vector2i center;

        private static bool cursorEnabled = false;

        /// <summary>
        /// Creates the MInput.
        /// </summary>
        /// <param name="game"></param>
        public static void Init(Game game)
        {
            MInput.game = game;

            infos = new Input.MouseKeyInfo[(int)Mouse.Button.ButtonCount];

            for (int i = 0; i < infos.Length; i++)
                infos[i] = new Input.MouseKeyInfo((Mouse.Button)i);
            
            center = new Vector2i(Settings.WindowSize.X / 2, Settings.WindowSize.Y / 2);

            ResetEnabled = true;
        }

        /// <summary>
        /// Updates the internal state of MInput, such as position, button pressed etc.
        /// </summary>
        public static void Update(GameTime gameTime)
        {
            oldMousePos = currentMousePos;
            currentMousePos = Mouse.GetPosition(game.window);

            for (int i = 0; i < infos.Length; i++)
                infos[i].Update(gameTime);
            
        }

        public static bool ResetEnabled
        {
            get;
            private set;
        }

        public static void ToggleResetEnabled()
        {
            ResetEnabled = !ResetEnabled;
        }

        public static void SetResetEnabled(bool value)
        {
            ResetEnabled = value;
        }

        public static Vector2f WindowPos()
        {
            return new Vector2f(currentMousePos.X, currentMousePos.Y);
        }

        public static Vector2f OldWindowPos()
        {
            return new Vector2f(oldMousePos.X, oldMousePos.Y);
        }

        public static Vector2f DirWindow()
        {
            return WindowPos() - OldWindowPos();
        }


        public static Vector2f GetWorldPos(RenderTarget target)
        {
            return target.MapPixelToCoords(Mouse.GetPosition(game.window));
        }

        public static void SetWorldPos(Vector2f pos, RenderTarget target)
        {
            Mouse.SetPosition(target.MapCoordsToPixel(pos));

           // return target.MapPixelToCoords(Mouse.GetPosition(game.window));
        }

        public static void ResetPos()
        {
            if (ResetEnabled)
            {
                oldMousePos = center;
                currentMousePos = center;
                Mouse.SetPosition(center, game.window);
            }
        }


        /// <summary>
        /// Checks if the mouse wheel is scrolled up.
        /// </summary>
        /// <returns>True if the mouse wheel was scrolled up this tick.</returns>
        public static bool MouseWheelUp()
        {
            return game.mouseWheel() == -1;
        }

        /// <summary>
        /// Checks if the mouse wheel was scrolled down.
        /// </summary>
        /// <returns>True if the mouse wheel was scrolled down this tick.</returns>
        public static bool MouseWheelDown()
        {
            return game.mouseWheel() == 1;
        }

        public static void SetWindowPos(Vector2i pos)
        {
            Mouse.SetPosition(pos, game.window);
        }

        public static bool CursorVisible
        {
            get { return cursorEnabled; }
            set { game.window.SetMouseCursorVisible(value); cursorEnabled = value;}
        }

        public static bool IsClicked(Mouse.Button button)
        {
            return infos[(int)button].IsClicked();
        }

        public static bool IsPressed(Mouse.Button button)
        {
            return infos[(int)button].IsPressed();
        }

        public static bool IsReleased(Mouse.Button button)
        {
            return infos[(int)button].IsReleased();
        }
    }

}