﻿using BobbleCrowd.GameStates;
using SFML.Graphics;
using SFML.Window;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BobbleCrowd.Message
{
    public class PauseMenu : AMessage
    {
        private Text[] texts;
        private Text title;

        private RectangleShape background;
        private bool firstCall = true;

        private int choosenText;

        public PauseMenu(Game game) : base(game)
        {
            background = new RectangleShape(new Vector2f(Settings.WindowSize.X, Settings.WindowSize.Y));
            background.FillColor = new Color(0, 0, 0, 128);

            texts = new Text[]{
                                    new Text("Resume", Game.Content.Load<Font>(Strings.FONT_NAME)),
                                    new Text("End", Game.Content.Load<Font>(Strings.FONT_NAME))};

            for (int i = 0; i < texts.Length; i++)
            {
                FloatRect localBounds = texts[i].GetLocalBounds();
                texts[i].Origin = new Vector2f(localBounds.Width * 0.5f, localBounds.Height * 0.5f);

                texts[i].Position = new Vector2f(background.Size.X * 0.5f, background.Size.Y * 0.5f + i * localBounds.Height * 2.0f - texts.Length * localBounds.Height * 1.25f);
            }

            title = new Text("Pause", Game.Content.Load<Font>(Strings.FONT_NAME));
            title.Scale = new Vector2f(1.5f, 1.5f);
            FloatRect titleBounds = title.GetLocalBounds();
            title.Origin = new Vector2f(titleBounds.Width * 0.5f, titleBounds.Height * 0.5f);
            title.Position = new Vector2f(background.Size.X * 0.5f, title.Scale.X * titleBounds.Height);

            this.Pauses = true;
            this.HasFinished = false;
        }

        public override void Update(GameTime gameTime, RenderTarget target)
        {
            if (firstCall)
                firstCall = false;

            else
            {

                if (KInput.IsClicked(Keyboard.Key.Escape))
                    HasFinished = true;

                Vector2f mousePos = MInput.WindowPos();

                ActivateButton(mousePos);

             
                if ((MInput.IsClicked(Mouse.Button.Left) && texts[choosenText].GetGlobalBounds().Contains(mousePos.X, mousePos.Y)) || KInput.IsClicked(Keyboard.Key.Return))
                {
                    if (choosenText == 0)
                        HasFinished = true;

                    else if (choosenText == 1)
                    {
                        game.SetGameState(new MainMenu());
                        HasFinished = true;
                    }
                     
                }
               
            }
        }

        private void ActivateButton(Vector2f mousePos)
        {

            int oldIndex = choosenText;

            for (int i = 0; i < texts.Length; i++)
            {
                if (texts[i].GetGlobalBounds().Contains(mousePos.X, mousePos.Y))
                    choosenText = i;        
            }

            if(KInput.IsClicked(Keyboard.Key.W))
                choosenText = (choosenText - 1 + texts.Length) % texts.Length;

            else if (KInput.IsClicked(Keyboard.Key.S))
                choosenText = (choosenText + 1 ) % texts.Length;
            

            texts[oldIndex].Color = Color.White;
            texts[choosenText].Color = Color.Red;

        }

        public override void Draw(GameTime gameTime, RenderTarget target)
        {
            target.Draw(background);

            foreach (Text t in texts)
                target.Draw(t);

            target.Draw(title);
        }

        public override void Init(GameTime gameTime, RenderTarget target)
        {
            firstCall = true;
            MInput.CursorVisible = true;
        }

        public override void ShutDown(GameTime gameTime, RenderTarget target)
        {
            MInput.CursorVisible = false;

        }
    }
}
