﻿using SFML.Graphics;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BobbleCrowd.Tiled
{
    /// <summary>
    /// Contains information about a TileSet.
    /// </summary>
    public class TileSet : IDisposable
    {
        public int Width
        {
            get;
            private set;
        }
        public int Height
        {
            get;
            private set;
        }

        public int TileWidth
        {
            get;
            private set;
        }
        public int TileHeight
        {
            get;
            private set;
        }

        public int NumTilesX
        {
            get;
            private set;
        }
        public int NumTilesY
        {
            get;
            private set;
        }

        public string Name
        {
            get;
            private set;
        }

        public Texture Texture
        {
            get;
            private set;
        }

        public int FirstId
        {
            get;
            private set;
        }
        

        public TileSet(String name, int tileWidth, int tileHeight, int firstId)
        {
            this.FirstId = firstId;
            this.Name = name;

            this.Texture = Game.Content.Load<Texture>(this.Name);

            this.Width = (int)this.Texture.Size.X;
            this.Height = (int)this.Texture.Size.Y;

            this.TileWidth = tileWidth;
            this.TileHeight = tileHeight;

            this.NumTilesX = this.Width / this.TileWidth;
            this.NumTilesY = this.Height / this.TileHeight;
        }


        /// <summary>
        /// Computes the Source Rectangle for the given id from this tileSet.
        /// </summary>
        /// <param name="id">The id ot the tile.</param>
        /// <returns></returns>
        public IntRect ToSourceRec(int id)
        {
            return new IntRect((id % NumTilesX) * TileWidth, (id / NumTilesX) * TileHeight, TileWidth, TileHeight);
        }

        public void Dispose()
        {
            Texture.Dispose();
        }
    }
}
