﻿using SFML.Graphics;
using SFML.Window;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BobbleCrowd.Tiled
{
    public class TileSetManager : IDisposable
    {
        private List<TileSet> tileSets = new List<TileSet>();
        private List<TileSetRange> tileRanges = new List<TileSetRange>();

        public TileSetManager()
        {

        }

        public IntRect GetSourceRectangle(int id)
        {
            int index = GetIndexFor(id);

            if (index == -1)
                return new IntRect();

            else
            {
                id -= tileRanges[index].min;

                return new IntRect( (id % tileSets[index].NumTilesX) * tileSets[index].TileWidth,
                                    (id / tileSets[index].NumTilesX) * tileSets[index].TileHeight,
                                    tileSets[index].TileWidth,
                                    tileSets[index].TileHeight);
            }       
        }

        public Texture GetTexture(int id)
        {
            int index = GetIndexFor(id);

            if (index == -1)
                return null;

            else 
                return tileSets[index].Texture;
        }

        public void AddTileSet(TileSet tileSet)
        {
            tileSets.Add(tileSet);

            int min = tileSet.FirstId;
            int max = tileSet.NumTilesX * tileSet.NumTilesY + min - 1;

            tileRanges.Add(new TileSetRange(min, max));
        }

        public int GetTileHeight(int id)
        {
            int index = GetIndexFor(id);

            if (index == -1)
                return 0;

            return tileSets[index].TileHeight;
        }

        public int GetTileWidth(int id)
        {
            int index = GetIndexFor(id);

            if (index == -1)
                return 0;

            return tileSets[index].TileWidth;
        }


        private int GetIndexFor(int tileId)
        {
            for (int i = 0; i < tileSets.Count; i++)
            {
                if (tileRanges[i].IsInRange(tileId))
                    return i;
            }

            return -1;
        }

        public void Dispose()
        {
            foreach (TileSet t in tileSets)
                t.Dispose();
        }


        private struct TileSetRange
        {
            public int min;
            public int max;

            public TileSetRange(int min, int max)
            {
                this.min = min;
                this.max = max;
            }

            public bool IsInRange(int id)
            {
                return min <= id && id <= max;
            }
        }
    }
}
