﻿using BobbleCrowd.Components;
using BobbleCrowd.Entities;
using BobbleCrowd.Tiled;
using BobbleCrowd.World;
using SFML.Graphics;
using SFML.Window;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BobbleCrowd
{
    class BobbleMixMode : GameMode
    {
        private const int       START_NUM_SPAWNS = 2;
        private const double    START_SPAWN_TIME = 5.0;
        private const int       START_NUMS = 0;

        public int numSpawns = START_NUM_SPAWNS;
        public double respawnTimeMax = START_SPAWN_TIME;
        public double respawnTime = START_SPAWN_TIME;
        
        public TargetBobbleThinker bobbleThinker;
        private MouseClicker mouseClicker;

        public Menu menu;

        public BobbleMixMode(GameWorld world) : base(world) { }

        public override void Update(GameTime gameTime)
        {
            double elapsed = gameTime.ElapsedTime.TotalSeconds;

            respawnTime -= elapsed;

            if (respawnTime <= 0)
            {
                respawnTime = respawnTimeMax;

                for (int i = 0; i < numSpawns; i++)
                    CreateBobble();

            }

            menu.Update(gameTime);
        }

        public override void PreChunkDraw(SFML.Graphics.RenderTarget target)
        {
            menu.Draw(target);

           
        }
        public override void PostChunkDraw(SFML.Graphics.RenderTarget target)
        {
            menu.MagicBobbleOverlayDraw(target);
            mouseClicker.Owner.Drawer.Draw(target);
        }

        public override void Init(LevelReader reader)
        {
            world.gameArea = new FloatRect(0, 64, Settings.WindowSize.X, Settings.WindowSize.Y - 64);
            TestCreateStuff();

        }
        private void TestCreateStuff()
        {
            menu = new Menu();
            menu.ini(world);

            CreateTargetBobble();

            Entity mouseClicker = new Entity("MouseClicker", new CircleCollider(4.0f), MInput.WindowPos());

            mouseClicker.Category = CollisionGroup.Group1;
            mouseClicker.Mask = CollisionGroup.Group2;
            this.mouseClicker = new MouseClicker();
            mouseClicker.Thinker = this.mouseClicker;

            mouseClicker.Drawer = new BasicTextureDrawer(Game.Content.Load<Texture>("gfx/crosshair.png"), true, true, 0.75f);

            mouseClicker.Response = new ClickResponse();

            world.AddEntity(mouseClicker);

            for (int i = 0; i < START_NUMS; i++)
                CreateBobble();


            bobbleThinker.Init();

        }

        private void CreateTargetBobble()
        {
            Bobble targetBobble = null;
            bobbleThinker = new TargetBobbleThinker();
            float radius = 38.0f;
            targetBobble = new Bobble("", new ConvexCollider(true, PolygonFactory.GetBox(64, 64)), new Vector2f((Settings.WindowSize.X) / 2, radius), new BobbleInfo(), 1.0f, false);
            
            targetBobble.Drawer.IsActive = false;
            targetBobble.Thinker = bobbleThinker;


            targetBobble.Response = new PushResponse(false, 0.5f);
            targetBobble.Mask = CollisionGroup.None;
            targetBobble.Category = CollisionGroup.None;
            targetBobble.CanCollide = false;

            (targetBobble.Drawer as BobbleDraw).SetBodyColor(targetBobble.info.color);

            world.AddEntity(targetBobble);

        }



        public void CreateBobble(BobbleInfo info)
        {
            float scale = 0.5f;
            float size = scale * 48.0f;

            Bobble bobble = new Bobble("Bobble", new CircleCollider(size), Vec2f.Zero, info, scale, true);
            float radius = bobble.Collider.Radius * 2.0f;

            bobble.Collider.Position = MathUtil.Rand.NextVec2f(world.gameArea.Left + radius, world.gameArea.Left + world.gameArea.Width - radius, world.gameArea.Top + radius, world.gameArea.Top + world.gameArea.Height - radius);

            bobbleThinker.AddInfo(info);
            world.AddEntity(bobble);

            bobble.Drawer.Update();
        }

 



        public void CreateBobble()
        {
            BobbleInfo info = new BobbleInfo();
            CreateBobble(info);


            bobbleThinker.TestChange();
        }

        public void RemoveBobble(Bobble bobble)
        {
            bobble.Kill();
            bobbleThinker.RemoveInfo(bobble.info);

            bobbleThinker.TestChange();
        }


        public override void ReceivedPoint()
        {

        }
    }
}
