﻿using BobbleCrowd.Components;
using BobbleCrowd.Entities;
using BobbleCrowd.Tiled;
using BobbleCrowd.World;
using SFML.Graphics;
using SFML.Window;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BobbleCrowd
{
    class BobbleSortMode : GameMode
    {
        private const int       NUM_STARTING_BOBBLES    = 0;
        private const int       NUM_SPAWNS_START        = 2;
        private const int       MORE_SPAWNS_AT          = 3;
        private const double    START_SPAWN_TIME        = 3.0;
        private const double    START_BOOM_TIME         = 20.0;
        private const int       DIFFICULTY_INCREASE_AT  = 10;
        private const double    PERCENT_FASTER_RESPAWN  = 0.05;
        private const double    PERCENT_FASTER_BOOM     = 0.025;

        private double  curBoomTime      = START_BOOM_TIME;
        private int     numSpawns       = NUM_SPAWNS_START;

        private double respawnTime      = 0;
        private double respawnTimeMax   = START_SPAWN_TIME;

        private MouseClicker clicker;
        public Menu menu;


        public BobbleSortMode(GameWorld world) 
            : base(world) 
        { 
        }
    


        public override void Update(GameTime gameTime)
        {
            double elapsed = gameTime.ElapsedTime.TotalSeconds;

            respawnTime -= elapsed;

            if (respawnTime <= 0)
            {
                respawnTime = respawnTimeMax;

                for (int i = 0; i < numSpawns; i++)
                    CreateBobble(curBoomTime);

            }

            
           menu.Update(gameTime);



           
        }

        public override void PreChunkDraw(SFML.Graphics.RenderTarget target)
        {
            menu.Draw(target);
        }

        public override void PostChunkDraw(SFML.Graphics.RenderTarget target)
        {
            clicker.Owner.Drawer.Draw(target);
        }

        public override void Init(LevelReader reader)
        {
            world.gameArea = new FloatRect(0, 64, Settings.WindowSize.X, Settings.WindowSize.Y - 285-64);
            menu = new Menu();
            menu.ini(world);

            for (int i = 0; i < reader.redZoneRect.Count; i++)
                world.AddEntity(world.CreateZone(reader.redZoneRect[i], BodyColor.Red));

            for (int i = 0; i < reader.blueZoneRect.Count; i++)
                world.AddEntity(world.CreateZone(reader.blueZoneRect[i], BodyColor.Blue));
            
            for (int i = 0; i < reader.greenZoneRect.Count; i++)
                world.AddEntity(world.CreateZone(reader.greenZoneRect[i],  BodyColor.Green));
       

            Entity mouseClicker = new Entity("MouseClicker", new CircleCollider(4.0f), MInput.WindowPos());
            mouseClicker.Category = CollisionGroup.Group1;
            mouseClicker.Mask = CollisionGroup.Group2;

            clicker = new MouseClicker();
            mouseClicker.Thinker = clicker; 
            mouseClicker.Drawer = new BasicTextureDrawer(Game.Content.Load<Texture>("gfx/crosshair.png"), true, true, 0.75f);
            mouseClicker.Response = new DragDropResponse();
            world.AddEntity(mouseClicker);


            for (int i = 0; i < NUM_STARTING_BOBBLES; i++)
            {
                CreateBobble(curBoomTime);
            }
        }


        private void CreateBobble(double lifeTime)
        {
            float scale = 0.5f;
            float size = scale * 48.0f;
            Bobble bobble = new Bobble("Bobble", new CircleCollider(size), new Vector2f(Settings.WindowSize.X / 2, 90), new BobbleInfo(), scale, true, lifeTime);
            float radius = bobble.Collider.Radius * 2.0f;
            bobble.Collider.Position = new Vector2f(Settings.WindowSize.X / 2, 90);

            world.AddEntity(bobble);
        }


        public override void ReceivedPoint()
        {
            if (menu.points % DIFFICULTY_INCREASE_AT == 0)
            {
                difficulty++;
                curBoomTime *= (1.0 - PERCENT_FASTER_BOOM);
                respawnTimeMax *= (1.0 - PERCENT_FASTER_RESPAWN);

                if (difficulty % MORE_SPAWNS_AT == 0)
                {
                    numSpawns++;
                }
            }
        }
    }
}
