﻿using BobbleCrowd.GameStates;
using BobbleCrowd.World;
using SFML.Graphics;
using SFML.Window;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BobbleCrowd
{
    public class Menu
    {
        private const double START_ROUND_TIME = 15000;
        private const double ADD_TIME_PER_POINT = 4000.0;
        private const double BAD_PERCENT = 1.5;
        private const double GOOD_PERCENT = 1.0;

        public GameWorld world;

        private Sprite bg;
        private Sprite feedOverlaySprite;
        private Sprite targetChangeOverlaySprite;
        public double roundTime;
        public int points;
        private Font fonts;
        private Text pointTxt;
        private Text pointsName;
        private Text timeTxt;
        private Text addPoint;
        private Text addTimeTxt;
        private bool addPointAktiv = false;
        private Text numMaster;

        private float run_FeedbackOverlayTime;
        private float max_FeedbackOverlayTime= 500;

        private float run_OverlayTime;
        private float max_OverlayTime = 500;

        public void Update(GameTime gameTime)
        {
            pointTxt.DisplayedString = points.ToString();
            pointTxt.Position = new Vector2f(Settings.WindowSize.X - 50 - pointTxt.GetLocalBounds().Width / 2, 12);
          //  points++;

            roundTime -= gameTime.ElapsedTime.TotalMilliseconds;

            if (run_OverlayTime > max_OverlayTime)
                run_OverlayTime = 0;
            else if (run_OverlayTime > 0)
                run_OverlayTime += (float)gameTime.ElapsedTime.TotalMilliseconds;


            double minutesF = (roundTime / 1000 / 60);
            int minutes = (int)minutesF;
            int seconds = (int)((minutesF - minutes) * 60);
            if (seconds > 9)
                timeTxt.DisplayedString = minutes + ":" + seconds;
            else
                timeTxt.DisplayedString = minutes + ":0" + seconds;
            if (addPointAktiv)
            {
                run_FeedbackOverlayTime += (float)gameTime.ElapsedTime.TotalMilliseconds;
                addPoint.Position = new Vector2f(addPoint.Position.X-0.5f, addPoint.Position.Y - 1);
                addTimeTxt.Position = new Vector2f(addTimeTxt.Position.X - 0.5f, addTimeTxt.Position.Y - 1);
                if (run_FeedbackOverlayTime>max_FeedbackOverlayTime)
                    addPointAktiv = false;

            }

            if((world.mode is BobbleMixMode) && roundTime <= 0)
            {
                this.roundTime = 0.0;

                this.world.GetInGame().GetParent().SetGameState(new HighscoreTable());
                this.world.endGame = true;
            }


        }
        public void Draw(SFML.Graphics.RenderTarget target)
        {
            target.Draw(bg);
            target.Draw(pointTxt);
            target.Draw(pointsName);


            if (world.mode is BobbleMixMode)
            {
                target.Draw(numMaster);
                target.Draw(timeTxt);
            }

            if (addPointAktiv)
            {
                target.Draw(addPoint);
                if (world.mode is BobbleMixMode)
                    target.Draw(addTimeTxt);
            }

        }
        public void MagicBobbleOverlayDraw(SFML.Graphics.RenderTarget target)
        {
            if (addPointAktiv)
            {

                feedOverlaySprite.Color = new Color(feedOverlaySprite.Color.R, feedOverlaySprite.Color.G, feedOverlaySprite.Color.B, (byte)((1-(run_FeedbackOverlayTime/max_FeedbackOverlayTime))*255));
                target.Draw(feedOverlaySprite);
            }
            else
            {
                run_FeedbackOverlayTime = 0;
            }
            if(run_OverlayTime>0)
            {
                float tmp = MathUtil.Clamp((1 - (run_OverlayTime / max_OverlayTime)),0,1);


                targetChangeOverlaySprite.Color = new Color(targetChangeOverlaySprite.Color.R, targetChangeOverlaySprite.Color.G, targetChangeOverlaySprite.Color.B, (byte)(tmp * 255));
                target.Draw(targetChangeOverlaySprite);
            }

        }
        public void ini(GameWorld world)
        {
            this.world = world;
            fonts = new Font(Game.Content.Load<Font>(Strings.FONT_NAME));
            bg = new Sprite(Game.Content.Load<Texture>("gfx/menu.png"));
            points = 0;
            pointTxt = new Text(points.ToString(), fonts);
            pointsName = new Text("Points: ", fonts);
            pointsName.Position = new Vector2f(Settings.WindowSize.X - 175, 12);

            roundTime = START_ROUND_TIME;
            
            double minutesF = (roundTime / 1000 / 60);
            int minutes = (int)minutesF;
            int seconds = (int)((minutesF - minutes) * 60);
            if(seconds>9)
                timeTxt = new Text(minutes+":"+seconds, fonts);
            else
                timeTxt = new Text(minutes + ":0" + seconds, fonts);
            timeTxt.Position = new Vector2f(32, 12);

            addPoint = new Text(10+"", fonts);
            addPoint.Position= new Vector2f(Settings.WindowSize.X - 50 - pointTxt.GetLocalBounds().Width / 2, 52);
            addPoint.Scale = new Vector2f(1.3f, 1.3f);
            addPoint.Style = Text.Styles.Bold;
            addTimeTxt = new Text("9001", fonts);
            addTimeTxt.Scale = new Vector2f(1.3f, 1.3f);
            addTimeTxt.Style = Text.Styles.Bold;
            numMaster = new Text("9001", fonts);
            numMaster.Position = new Vector2f((Settings.WindowSize.X / 2)+30, 12);
            feedOverlaySprite = new Sprite(Game.Content.Load<Texture>("gfx/error.png"));
            targetChangeOverlaySprite = new Sprite(Game.Content.Load<Texture>("gfx/splash.png"));
            run_FeedbackOverlayTime = 0;    
        }

        public void AddPoints(int deltaPoint)
        {
            addPointAktiv = true;
            addPoint.Position = new Vector2f(Settings.WindowSize.X - 42 - pointTxt.GetLocalBounds().Width / 2, 52);
            addTimeTxt.Position = new Vector2f(70, 52);
            run_FeedbackOverlayTime = 0;

            if (deltaPoint < 0)
            {

               addPoint.Color = Color.Red;
                addTimeTxt.Color = Color.Red;
                addPoint.Position = new Vector2f(Settings.WindowSize.X - 42 - pointTxt.GetLocalBounds().Width / 2, -40);
                addTimeTxt.DisplayedString = "-" + ((ADD_TIME_PER_POINT * BAD_PERCENT) / 1000.0);
                feedOverlaySprite.Color = Color.Red;
                roundTime -= (ADD_TIME_PER_POINT * BAD_PERCENT);

            }
            else
            {
                

                addPoint.Color = Color.Green;
                addTimeTxt.Color = Color.Green;
                feedOverlaySprite.Color = Color.Green;
                addPoint.DisplayedString = "+" + deltaPoint.ToString();
                addTimeTxt.DisplayedString = "+" + ((ADD_TIME_PER_POINT * GOOD_PERCENT) / 1000.0);

                roundTime += (ADD_TIME_PER_POINT * GOOD_PERCENT);

                this.points += deltaPoint;
                world.mode.ReceivedPoint();
            }

          

            if (points < 0)
                points = 0;


        }
        
        public void targetBobbleChanged()
        {
            run_OverlayTime += 0.001f;
        }
        public void SetNumberOfCurrent(int current)
        {
            numMaster.DisplayedString="x"+current + "";
        }

        
    }
}
